package etf

import (
	"bytes"
	"encoding/binary"
	"io"
	"math"
	"math/big"
	"math/rand"
	"testing"
	"time"
)

func BenchmarkReadAtom(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand.Seed(time.Now().UnixNano())
	max := 64
	length := 64

	buf := new(bytes.Buffer)

	for i := 0; i < max; i++ {
		s := bytes.Repeat([]byte{'a'}, length)
		b := bytes.Map(randRune, s)
		buf.Write([]byte{ettSmallAtom, byte(length)})
		buf.Write(b)
	}

	decoder := c.NewDecoder(buf)
	b.StartTimer()

	for i := 0; i < b.N; i++ {
		_, err := decoder.NextTerm()

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkReadBigInt(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand := rand.New(rand.NewSource(time.Now().UnixNano()))
	uint64Max := big.NewInt(math.MaxInt64)
	top := new(big.Int).Mul(uint64Max, uint64Max)
	max := 512

	w := new(bytes.Buffer)
	for i := 0; i < max; i++ {
		a := new(big.Int).Rand(rand, top)
		b := new(big.Int).Rand(rand, top)
		v := new(big.Int).Sub(a, b)
		sign := 0
		if v.Sign() < 0 {
			sign = 1
		}
		bytes := reverse(new(big.Int).Abs(a).Bytes())
		w.Write([]byte{ettSmallBig, byte(len(bytes)), byte(sign)})
		w.Write(bytes)
	}

	decoder := c.NewDecoder(w)
	b.StartTimer()

	for i := 0; i < b.N; i++ {
		_, err := decoder.NextTerm()

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkReadBinary(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand.Seed(time.Now().UnixNano())
	max := 64
	length := 64
	binaries := make([]*bytes.Buffer, max)

	for i := 0; i < max; i++ {
		w := new(bytes.Buffer)
		s := bytes.Repeat([]byte{'a'}, length)
		b := bytes.Map(func(rune) rune { return rune(byte(rand.Int())) }, s)
		w.Write([]byte{ettBinary})
		binary.Write(w, binary.BigEndian, uint32(len(b)))
		w.Write(b)
		binaries[i] = w
	}

	b.StartTimer()

	for i := 0; i < b.N; i++ {
		in := binaries[i%max]
		_, err := c.Read(in)

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkReadFloat(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand.Seed(time.Now().UnixNano())
	max := 512
	floats := make([]*bytes.Buffer, max)

	for i := 0; i < max; i++ {
		w := new(bytes.Buffer)
		v := rand.ExpFloat64() - rand.ExpFloat64()
		w.Write([]byte{ettNewFloat})
		binary.Write(w, binary.BigEndian, math.Float64bits(v))
		floats[i] = w
	}

	b.StartTimer()

	for i := 0; i < b.N; i++ {
		in := floats[i%max]
		_, err := c.Read(in)

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkReadPid(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand.Seed(time.Now().UnixNano())
	max := 64
	length := 16
	pids := make([]*bytes.Buffer, max)

	for i := 0; i < max; i++ {
		w := new(bytes.Buffer)
		s := bytes.Repeat([]byte{'a'}, length)
		b := bytes.Map(randRune, s)
		b[6] = '@'
		w.Write([]byte{ettPid, ettSmallAtom, byte(length)})
		w.Write(b)
		w.Write([]byte{0, 0, 0, uint8(rand.Int())})
		w.Write([]byte{0, 0, 0, uint8(rand.Int())})
		w.Write([]byte{uint8(rand.Int())})
		pids[i] = w
	}

	b.StartTimer()

	for i := 0; i < b.N; i++ {
		in := pids[i%max]
		_, err := c.Read(in)

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkReadString(b *testing.B) {
	b.StopTimer()
	c := new(Context)

	rand.Seed(time.Now().UnixNano())
	max := 64
	length := 64
	strings := make([]*bytes.Buffer, max)

	for i := 0; i < max; i++ {
		w := new(bytes.Buffer)
		s := bytes.Repeat([]byte{'a'}, length)
		b := bytes.Map(randRune, s)
		w.Write([]byte{ettString})
		binary.Write(w, binary.BigEndian, uint16(len(b)))
		w.Write(b)
		strings[i] = w
	}

	b.StartTimer()

	for i := 0; i < b.N; i++ {
		in := strings[i%max]
		_, err := c.Read(in)

		if err != io.EOF && err != nil {
			b.Fatal(err)
		}
	}
}

func randRune(_ rune) rune {
	return rune('0' + byte(rand.Intn('z'-'0'+1)))
}

func BenchmarkRealistic(b *testing.B) {
	c := new(Context)
	c.ConvertBinaryToString = true

	buf := new(bytes.Buffer)
	for i := 0; i < b.N; i++ {
		buf.Write(realisticEncoded)
	}

	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		decoder := c.NewDecoder(buf)
		_, err := decoder.NextTerm()
		if err != nil {
			b.Fatal(err)
		}
	}
}

// func BenchmarkRealisticJson(b *testing.B) {

// 	for i := 0; i < b.N; i++ {
// 		var dest map[string]interface{}
// 		err := json.Unmarshal(realisticJson, &dest)
// 		if err != nil {
// 			b.Fatal(err)
// 		}
// 	}
// }

var (
	realisticEncoded = []byte{116, 0, 0, 0, 4, 100, 0, 1, 100, 116, 0, 0, 0, 24, 100, 0, 14, 97, 102, 107, 95, 99, 104, 97, 110, 110, 101, 108, 95, 105, 100, 110, 8, 0, 1, 0, 4, 235, 18, 95, 202, 3, 100, 0, 11, 97, 102, 107, 95, 116, 105, 109, 101, 111, 117, 116, 98, 0, 0, 1, 44, 100, 0, 14, 97, 112, 112, 108, 105, 99, 97, 116, 105, 111, 110, 95, 105, 100, 100, 0, 3, 110, 105, 108, 100, 0, 8, 99, 104, 97, 110, 110, 101, 108, 115, 108, 0, 0, 0, 11, 116, 0, 0, 0, 9, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 10, 0, 0, 49, 223, 173, 86, 4, 100, 0, 18, 108, 97, 115, 116, 95, 112, 105, 110, 95, 116, 105, 109, 101, 115, 116, 97, 109, 112, 109, 0, 0, 0, 32, 50, 48, 49, 54, 45, 48, 57, 45, 49, 48, 84, 48, 57, 58, 50, 52, 58, 49, 57, 46, 53, 56, 48, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 97, 115, 100, 97, 115, 100, 97, 97, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 108, 0, 0, 0, 2, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 0, 64, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 4, 114, 111, 108, 101, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 97, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 35, 126, 145, 58, 3, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 6, 109, 101, 109, 98, 101, 114, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 4, 100, 0, 5, 116, 111, 112, 105, 99, 109, 0, 0, 0, 3, 65, 65, 65, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 0, 197, 33, 123, 72, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 0, 0, 132, 19, 157, 53, 88, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 15, 97, 110, 111, 116, 104, 101, 114, 95, 99, 104, 97, 110, 110, 101, 108, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 108, 0, 0, 0, 1, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 2, 0, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 11, 0, 0, 239, 215, 247, 193, 3, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 114, 111, 108, 101, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 0, 100, 0, 5, 116, 111, 112, 105, 99, 109, 0, 0, 0, 0, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 127, 217, 111, 89, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 0, 0, 2, 134, 208, 201, 50, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 3, 97, 97, 97, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 1, 100, 0, 5, 116, 111, 112, 105, 99, 109, 0, 0, 0, 0, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 192, 88, 6, 71, 134, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 0, 0, 66, 169, 114, 12, 208, 3, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 6, 109, 111, 100, 108, 111, 103, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 2, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 192, 131, 140, 175, 148, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 0, 0, 4, 127, 79, 101, 43, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 5, 104, 101, 108, 108, 111, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 108, 0, 0, 0, 1, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 0, 4, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 4, 114, 111, 108, 101, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 3, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 196, 197, 125, 168, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 1, 0, 4, 71, 14, 174, 86, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 7, 116, 101, 115, 116, 100, 101, 110, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 108, 0, 0, 0, 2, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 0, 64, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 114, 111, 108, 101, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 0, 12, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 35, 126, 145, 58, 3, 100, 0, 4, 116, 121, 112, 101, 100, 0, 6, 109, 101, 109, 98, 101, 114, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 5, 100, 0, 5, 116, 111, 112, 105, 99, 109, 0, 0, 0, 13, 97, 115, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 66, 20, 23, 123, 183, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 20, 0, 64, 106, 54, 123, 183, 3, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 13, 116, 104, 101, 45, 98, 101, 101, 45, 109, 111, 118, 105, 101, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 6, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 64, 0, 166, 148, 189, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 1, 0, 66, 66, 19, 217, 69, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 4, 101, 114, 114, 115, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 7, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 10, 0, 192, 202, 168, 148, 189, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 110, 8, 0, 0, 0, 132, 182, 220, 211, 69, 4, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 5, 106, 111, 105, 110, 115, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 8, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 116, 0, 0, 0, 8, 100, 0, 7, 98, 105, 116, 114, 97, 116, 101, 98, 0, 0, 250, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 4, 235, 18, 95, 202, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 3, 97, 115, 100, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 0, 100, 0, 4, 116, 121, 112, 101, 100, 0, 5, 118, 111, 105, 99, 101, 100, 0, 10, 117, 115, 101, 114, 95, 108, 105, 109, 105, 116, 97, 0, 116, 0, 0, 0, 8, 100, 0, 2, 105, 100, 110, 8, 0, 2, 0, 132, 124, 220, 83, 245, 3, 100, 0, 10, 105, 115, 95, 112, 114, 105, 118, 97, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 15, 108, 97, 115, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 105, 100, 100, 0, 3, 110, 105, 108, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 5, 100, 100, 100, 100, 100, 100, 0, 21, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 95, 111, 118, 101, 114, 119, 114, 105, 116, 101, 115, 108, 0, 0, 0, 3, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 98, 0, 0, 4, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 61, 36, 235, 22, 4, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 4, 114, 111, 108, 101, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 97, 0, 100, 0, 4, 100, 101, 110, 121, 97, 0, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 4, 114, 111, 108, 101, 116, 0, 0, 0, 4, 100, 0, 5, 97, 108, 108, 111, 119, 97, 0, 100, 0, 4, 100, 101, 110, 121, 98, 0, 0, 12, 0, 100, 0, 2, 105, 100, 110, 8, 0, 2, 0, 192, 206, 43, 235, 22, 4, 100, 0, 4, 116, 121, 112, 101, 109, 0, 0, 0, 4, 114, 111, 108, 101, 106, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 9, 100, 0, 5, 116, 111, 112, 105, 99, 100, 0, 3, 110, 105, 108, 100, 0, 4, 116, 121, 112, 101, 100, 0, 4, 116, 101, 120, 116, 106, 100, 0, 29, 100, 101, 102, 97, 117, 108, 116, 95, 109, 101, 115, 115, 97, 103, 101, 95, 110, 111, 116, 105, 102, 105, 99, 97, 116, 105, 111, 110, 115, 97, 0, 100, 0, 6, 101, 109, 111, 106, 105, 115, 106, 100, 0, 23, 101, 120, 112, 108, 105, 99, 105, 116, 95, 99, 111, 110, 116, 101, 110, 116, 95, 102, 105, 108, 116, 101, 114, 97, 2, 100, 0, 8, 102, 101, 97, 116, 117, 114, 101, 115, 106, 100, 0, 4, 105, 99, 111, 110, 109, 0, 0, 0, 32, 50, 48, 56, 52, 57, 50, 99, 49, 99, 51, 100, 56, 101, 54, 100, 50, 99, 50, 52, 98, 101, 98, 54, 53, 55, 54, 57, 55, 57, 98, 56, 99, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 53, 45, 49, 51, 84, 50, 48, 58, 48, 55, 58, 48, 49, 46, 57, 48, 54, 54, 57, 57, 43, 48, 48, 58, 48, 48, 100, 0, 5, 108, 97, 114, 103, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 12, 109, 101, 109, 98, 101, 114, 95, 99, 111, 117, 110, 116, 97, 10, 100, 0, 7, 109, 101, 109, 98, 101, 114, 115, 108, 0, 0, 0, 10, 116, 0, 0, 0, 6, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 54, 45, 48, 53, 45, 50, 48, 84, 50, 51, 58, 51, 53, 58, 52, 57, 46, 48, 52, 54, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 105, 99, 107, 109, 0, 0, 0, 3, 51, 51, 51, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 3, 110, 8, 0, 1, 0, 192, 249, 248, 85, 100, 3, 110, 8, 0, 0, 0, 194, 161, 234, 59, 103, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 4, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 55, 56, 55, 56, 50, 51, 102, 97, 99, 57, 54, 49, 102, 50, 55, 56, 97, 55, 102, 54, 52, 53, 100, 57, 97, 99, 55, 101, 55, 49, 57, 50, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 51, 49, 50, 52, 100, 0, 2, 105, 100, 110, 8, 0, 0, 224, 0, 251, 37, 196, 118, 1, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 8, 74, 111, 110, 97, 115, 55, 52, 55, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 54, 45, 49, 48, 45, 50, 55, 84, 50, 50, 58, 52, 55, 58, 51, 54, 46, 50, 56, 51, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 4, 110, 8, 0, 0, 0, 130, 25, 68, 98, 89, 3, 110, 8, 0, 0, 0, 194, 161, 234, 59, 103, 3, 110, 8, 0, 0, 0, 194, 202, 154, 160, 179, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 53, 97, 101, 98, 54, 56, 99, 50, 57, 98, 53, 54, 98, 51, 100, 57, 50, 101, 100, 100, 98, 54, 102, 52, 54, 100, 102, 53, 48, 53, 49, 99, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 51, 56, 54, 49, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 184, 74, 51, 39, 2, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 4, 68, 121, 110, 111, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 49, 45, 49, 52, 84, 48, 48, 58, 51, 55, 58, 51, 55, 46, 57, 55, 54, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 2, 110, 8, 0, 0, 0, 128, 16, 117, 231, 189, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 101, 53, 98, 49, 99, 50, 102, 99, 57, 100, 97, 99, 53, 52, 99, 101, 54, 100, 57, 50, 50, 50, 53, 98, 56, 98, 50, 55, 52, 100, 48, 100, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 54, 48, 51, 48, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 2, 4, 194, 254, 80, 2, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 13, 83, 105, 114, 66, 114, 111, 66, 111, 116, 32, 226, 154, 148, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 54, 45, 49, 49, 45, 49, 57, 84, 48, 50, 58, 53, 54, 58, 53, 53, 46, 48, 56, 49, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 5, 110, 8, 0, 0, 0, 194, 161, 234, 59, 103, 3, 110, 8, 0, 10, 0, 0, 77, 78, 238, 117, 3, 110, 8, 0, 4, 0, 66, 211, 83, 247, 132, 3, 110, 8, 0, 0, 0, 194, 202, 154, 160, 179, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 102, 53, 102, 54, 53, 55, 53, 53, 102, 54, 55, 97, 101, 49, 100, 99, 56, 56, 100, 57, 98, 98, 50, 55, 49, 100, 48, 102, 53, 98, 101, 102, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 56, 55, 57, 50, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 126, 40, 19, 99, 2, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 9, 84, 97, 116, 115, 117, 109, 97, 107, 105, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 49, 45, 49, 55, 84, 48, 52, 58, 50, 51, 58, 48, 56, 46, 57, 55, 57, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 2, 110, 8, 0, 11, 0, 0, 239, 215, 247, 193, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 102, 99, 100, 102, 98, 100, 99, 52, 51, 97, 50, 97, 51, 101, 56, 51, 54, 53, 57, 57, 98, 48, 99, 55, 56, 97, 52, 99, 49, 55, 97, 51, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 56, 55, 54, 48, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 64, 148, 10, 169, 213, 2, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 10, 89, 65, 71, 80, 68, 66, 46, 120, 121, 122, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 49, 45, 48, 54, 84, 49, 52, 58, 48, 56, 58, 53, 52, 46, 55, 57, 50, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 2, 110, 8, 0, 0, 0, 194, 202, 154, 160, 179, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 101, 98, 55, 50, 53, 50, 49, 51, 57, 97, 100, 49, 53, 102, 52, 51, 55, 57, 55, 48, 48, 102, 55, 100, 52, 101, 53, 99, 98, 51, 100, 99, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 48, 54, 53, 48, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 0, 83, 218, 63, 0, 3, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 6, 65, 100, 111, 110, 105, 115, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 53, 45, 49, 50, 84, 49, 54, 58, 51, 51, 58, 52, 51, 46, 49, 50, 52, 49, 49, 56, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 100, 0, 3, 110, 105, 108, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 54, 56, 55, 52, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 2, 83, 140, 97, 42, 3, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 6, 102, 117, 110, 98, 111, 116, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 49, 45, 48, 54, 84, 48, 48, 58, 52, 53, 58, 49, 56, 46, 49, 51, 49, 48, 48, 48, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 2, 110, 8, 0, 13, 0, 130, 91, 126, 156, 179, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 53, 100, 51, 53, 56, 51, 49, 49, 55, 57, 99, 102, 57, 98, 98, 54, 53, 98, 101, 48, 48, 52, 98, 101, 99, 56, 48, 52, 53, 98, 98, 54, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 49, 49, 54, 54, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 35, 126, 145, 58, 3, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 14, 89, 65, 71, 80, 68, 66, 32, 116, 101, 115, 116, 105, 110, 103, 116, 0, 0, 0, 5, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 53, 45, 49, 51, 84, 50, 48, 58, 48, 55, 58, 48, 49, 46, 57, 48, 54, 54, 57, 57, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 5, 114, 111, 108, 101, 115, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 100, 0, 3, 110, 105, 108, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 57, 55, 57, 49, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 194, 66, 40, 222, 185, 3, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 7, 78, 111, 116, 65, 66, 111, 116, 116, 0, 0, 0, 6, 100, 0, 4, 100, 101, 97, 102, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 9, 106, 111, 105, 110, 101, 100, 95, 97, 116, 109, 0, 0, 0, 32, 50, 48, 49, 55, 45, 48, 52, 45, 48, 57, 84, 48, 48, 58, 48, 49, 58, 48, 49, 46, 53, 53, 52, 52, 49, 49, 43, 48, 48, 58, 48, 48, 100, 0, 4, 109, 117, 116, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 105, 99, 107, 100, 0, 3, 110, 105, 108, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 2, 110, 8, 0, 0, 0, 66, 147, 91, 162, 164, 3, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 106, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 5, 100, 0, 6, 97, 118, 97, 116, 97, 114, 109, 0, 0, 0, 32, 101, 49, 52, 54, 54, 56, 51, 51, 54, 99, 55, 51, 98, 100, 98, 102, 102, 50, 99, 50, 57, 97, 50, 56, 53, 100, 98, 102, 49, 51, 54, 57, 100, 0, 3, 98, 111, 116, 100, 0, 4, 116, 114, 117, 101, 100, 0, 13, 100, 105, 115, 99, 114, 105, 109, 105, 110, 97, 116, 111, 114, 109, 0, 0, 0, 4, 53, 52, 54, 51, 100, 0, 2, 105, 100, 110, 8, 0, 22, 0, 128, 183, 187, 77, 43, 4, 100, 0, 8, 117, 115, 101, 114, 110, 97, 109, 101, 109, 0, 0, 0, 7, 68, 69, 109, 112, 105, 114, 101, 106, 100, 0, 9, 109, 102, 97, 95, 108, 101, 118, 101, 108, 97, 0, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 15, 116, 117, 117, 117, 117, 117, 115, 32, 116, 105, 110, 103, 103, 103, 97, 100, 0, 8, 111, 119, 110, 101, 114, 95, 105, 100, 110, 8, 0, 0, 224, 0, 251, 37, 196, 118, 1, 100, 0, 9, 112, 114, 101, 115, 101, 110, 99, 101, 115, 108, 0, 0, 0, 8, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 100, 0, 3, 110, 105, 108, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 224, 0, 251, 37, 196, 118, 1, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 116, 0, 0, 0, 1, 109, 0, 0, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 19, 100, 121, 110, 111, 98, 111, 116, 46, 110, 101, 116, 32, 124, 32, 63, 104, 101, 108, 112, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 184, 74, 51, 39, 2, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 116, 0, 0, 0, 3, 109, 0, 0, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 13, 83, 97, 121, 32, 63, 99, 111, 109, 109, 97, 110, 100, 115, 109, 0, 0, 0, 4, 116, 121, 112, 101, 97, 1, 109, 0, 0, 0, 3, 117, 114, 108, 109, 0, 0, 0, 39, 104, 116, 116, 112, 115, 58, 47, 47, 119, 119, 119, 46, 116, 119, 105, 116, 99, 104, 46, 116, 118, 47, 83, 105, 114, 66, 114, 111, 66, 111, 116, 47, 112, 114, 111, 102, 105, 108, 101, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 2, 4, 194, 254, 80, 2, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 116, 0, 0, 0, 3, 109, 0, 0, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 10, 85, 115, 101, 32, 116, 33, 104, 101, 108, 112, 109, 0, 0, 0, 4, 116, 121, 112, 101, 97, 1, 109, 0, 0, 0, 3, 117, 114, 108, 109, 0, 0, 0, 21, 104, 116, 116, 112, 115, 58, 47, 47, 116, 97, 116, 115, 117, 109, 97, 107, 105, 46, 120, 121, 122, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 126, 40, 19, 99, 2, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 116, 0, 0, 0, 3, 109, 0, 0, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 19, 118, 48, 46, 49, 57, 46, 55, 32, 72, 121, 103, 105, 101, 110, 105, 99, 32, 58, 41, 109, 0, 0, 0, 4, 116, 121, 112, 101, 97, 0, 109, 0, 0, 0, 3, 117, 114, 108, 109, 0, 0, 0, 0, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 64, 148, 10, 169, 213, 2, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 116, 0, 0, 0, 1, 109, 0, 0, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 23, 97, 33, 104, 101, 108, 112, 32, 226, 151, 143, 32, 51, 51, 53, 56, 32, 83, 101, 114, 118, 101, 114, 115, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 0, 83, 218, 63, 0, 3, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 100, 0, 3, 110, 105, 108, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 2, 83, 140, 97, 42, 3, 116, 0, 0, 0, 3, 100, 0, 4, 103, 97, 109, 101, 100, 0, 3, 110, 105, 108, 100, 0, 6, 115, 116, 97, 116, 117, 115, 100, 0, 6, 111, 110, 108, 105, 110, 101, 100, 0, 4, 117, 115, 101, 114, 116, 0, 0, 0, 1, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 194, 66, 40, 222, 185, 3, 106, 100, 0, 6, 114, 101, 103, 105, 111, 110, 109, 0, 0, 0, 7, 117, 115, 45, 101, 97, 115, 116, 100, 0, 5, 114, 111, 108, 101, 115, 108, 0, 0, 0, 21, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 194, 9, 237, 110, 139, 2, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 9, 64, 101, 118, 101, 114, 121, 111, 110, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 0, 0, 12, 0, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 0, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 2, 57, 175, 29, 146, 2, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 4, 77, 101, 101, 54, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 3, 247, 252, 127, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 15, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 25, 68, 98, 89, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 4, 116, 114, 117, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 4, 68, 121, 110, 111, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 28, 52, 108, 119, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 21, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 192, 249, 248, 85, 100, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 4, 109, 117, 116, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 6, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 17, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 98, 0, 52, 152, 219, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 194, 161, 234, 59, 103, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 4, 116, 114, 117, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 6, 109, 101, 109, 98, 101, 114, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 16, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 10, 0, 0, 77, 78, 238, 117, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 4, 116, 114, 117, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 9, 84, 97, 116, 115, 117, 109, 97, 107, 105, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 0, 197, 44, 79, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 14, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 4, 0, 66, 211, 83, 247, 132, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 9, 115, 116, 114, 101, 97, 109, 105, 110, 103, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 6, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 12, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 130, 190, 17, 102, 133, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 4, 116, 114, 117, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 9, 64, 101, 118, 101, 114, 121, 111, 110, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 11, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 98, 0, 31, 139, 76, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 66, 147, 91, 162, 164, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 10, 77, 79, 100, 101, 114, 108, 97, 116, 111, 114, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 20, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 13, 0, 130, 91, 126, 156, 179, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 4, 116, 114, 117, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 14, 89, 65, 71, 80, 68, 66, 32, 116, 101, 115, 116, 105, 110, 103, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 31, 247, 236, 87, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 5, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 194, 202, 154, 160, 179, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 115, 116, 114, 101, 97, 109, 101, 114, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 13, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 11, 0, 130, 255, 161, 160, 179, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 11, 98, 97, 100, 115, 116, 114, 101, 97, 109, 101, 114, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 10, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 128, 16, 117, 231, 189, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 4, 116, 114, 117, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 13, 83, 105, 114, 66, 114, 111, 66, 111, 116, 32, 226, 154, 148, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 12, 55, 252, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 9, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 11, 0, 0, 239, 215, 247, 193, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 4, 116, 114, 117, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 10, 89, 65, 71, 80, 68, 66, 46, 120, 121, 122, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 0, 51, 204, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 8, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 3, 0, 66, 161, 215, 94, 227, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 110, 101, 119, 32, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 7, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 128, 151, 216, 94, 227, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 110, 101, 119, 32, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 6, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 0, 244, 226, 100, 227, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 110, 101, 119, 32, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 4, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 1, 0, 4, 64, 232, 100, 227, 3, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 8, 110, 101, 119, 32, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 65, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 3, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 0, 0, 130, 61, 36, 235, 22, 4, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 11, 109, 101, 109, 98, 101, 114, 45, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 1, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 2, 0, 192, 206, 43, 235, 22, 4, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 9, 109, 117, 116, 101, 45, 114, 111, 108, 101, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 98, 22, 53, 92, 1, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 2, 116, 0, 0, 0, 8, 100, 0, 5, 99, 111, 108, 111, 114, 97, 0, 100, 0, 5, 104, 111, 105, 115, 116, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 2, 105, 100, 110, 8, 0, 2, 0, 194, 0, 175, 101, 26, 4, 100, 0, 7, 109, 97, 110, 97, 103, 101, 100, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 11, 109, 101, 110, 116, 105, 111, 110, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 4, 110, 97, 109, 101, 109, 0, 0, 0, 3, 97, 115, 100, 100, 0, 11, 112, 101, 114, 109, 105, 115, 115, 105, 111, 110, 115, 97, 0, 100, 0, 8, 112, 111, 115, 105, 116, 105, 111, 110, 97, 1, 106, 100, 0, 6, 115, 112, 108, 97, 115, 104, 100, 0, 3, 110, 105, 108, 100, 0, 11, 117, 110, 97, 118, 97, 105, 108, 97, 98, 108, 101, 100, 0, 5, 102, 97, 108, 115, 101, 100, 0, 18, 118, 101, 114, 105, 102, 105, 99, 97, 116, 105, 111, 110, 95, 108, 101, 118, 101, 108, 97, 0, 100, 0, 12, 118, 111, 105, 99, 101, 95, 115, 116, 97, 116, 101, 115, 106, 100, 0, 2, 111, 112, 97, 0, 100, 0, 1, 115, 97, 2, 100, 0, 1, 116, 100, 0, 12, 71, 85, 73, 76, 68, 95, 67, 82, 69, 65, 84, 69}
	// realisticJson    = []byte(`{"voice_states":[],"verification_level":0,"unavailable":false,"splash":null,"roles":[{"position":0,"permissions":3072,"name":"@everyone","mentionable":false,"managed":false,"id":"183362174188650497","hoist":false,"color":0},{"position":15,"permissions":66583679,"name":"Mee6","mentionable":false,"managed":false,"id":"185243173088526336","hoist":false,"color":0},{"position":21,"permissions":473197687,"name":"Dyno","mentionable":false,"managed":true,"id":"241332099666280448","hoist":false,"color":0},{"position":17,"permissions":104160257,"name":"mute","mentionable":false,"managed":false,"id":"244414807615209473","hoist":false,"color":0},{"position":16,"permissions":372595713,"name":"member","mentionable":true,"managed":false,"id":"245230583637213184","hoist":false,"color":3447003},{"position":14,"permissions":12921935,"name":"Tatsumaki","mentionable":false,"managed":true,"id":"249367374455635978","hoist":false,"color":0},{"position":12,"permissions":104160257,"name":"streaming","mentionable":false,"managed":false,"id":"253599418438254596","hoist":false,"color":0},{"position":11,"permissions":372595713,"name":"@everyone","mentionable":true,"managed":false,"id":"253721180412968961","hoist":false,"color":0},{"position":20,"permissions":372595777,"name":"MOderlator","mentionable":false,"managed":false,"id":"262513192490631168","hoist":false,"color":2067276},{"position":5,"permissions":536341591,"name":"YAGPDB testing","mentionable":false,"managed":true,"id":"266728869460049933","hoist":false,"color":0},{"position":13,"permissions":372595777,"name":"streamer","mentionable":false,"managed":false,"id":"266733389632110592","hoist":false,"color":0},{"position":10,"permissions":372595713,"name":"badstreamer","mentionable":false,"managed":false,"id":"266733420581879819","hoist":false,"color":0},{"position":9,"permissions":204995649,"name":"SirBroBot ⚔","mentionable":false,"managed":true,"id":"269626042686111744","hoist":false,"color":0},{"position":8,"permissions":3394625,"name":"YAGPDB.xyz","mentionable":false,"managed":true,"id":"270769959418724363","hoist":false,"color":0},{"position":7,"permissions":372595777,"name":"new role","mentionable":false,"managed":false,"id":"280171882043539459","hoist":false,"color":0},{"position":6,"permissions":372595777,"name":"new role","mentionable":false,"managed":false,"id":"280171886174797824","hoist":false,"color":0},{"position":4,"permissions":372595777,"name":"new role","mentionable":false,"managed":false,"id":"280178527746129921","hoist":false,"color":0},{"position":3,"permissions":372595777,"name":"new role","mentionable":false,"managed":false,"id":"280178550496296961","hoist":false,"color":0},{"position":1,"permissions":372595713,"name":"member-role","mentionable":false,"managed":false,"id":"294681366522626048","hoist":false,"color":0},{"position":2,"permissions":372595713,"name":"mute-role","mentionable":false,"managed":false,"id":"294681399024156674","hoist":false,"color":0},{"position":1,"permissions":0,"name":"asd","mentionable":false,"managed":false,"id":"295660527852584962","hoist":false,"color":0}],"region":"us-east","presences":[{"user":{"id":"105487308693757952"},"status":"online","game":null},{"user":{"id":"155149108183695360"},"status":"online","game":{"name":"dynobot.net | ?help"}},{"user":{"id":"166913295457058817"},"status":"online","game":{"url":"https://www.twitch.tv/SirBroBot/profile","type":1,"name":"Say ?commands"}},{"user":{"id":"172002275412279296"},"status":"online","game":{"url":"https://tatsumaki.xyz","type":1,"name":"98287 users"}},{"user":{"id":"204255221017214977"},"status":"online","game":{"url":"","type":0,"name":"v0.19.7 Hygienic :)"}},{"user":{"id":"216242989041713152"},"status":"online","game":{"name":"a!help ● 3356 Servers"}},{"user":{"id":"228101986451587072"},"status":"online","game":null},{"user":{"id":"268489917305323520"},"status":"online","game":null}],"owner_id":"105487308693757952","name":"tuuuuus tinggga","mfa_level":0,"members":[{"user":{"username":"Jonas747","id":"105487308693757952","discriminator":"3124","avatar":"787823fac961f278a7f645d9ac7e7192"},"roles":["244414807615209473","245230583637213184","280178550496296961"],"nick":"333","mute":false,"joined_at":"2016-05-20T23:35:49.046000+00:00","deaf":false},{"user":{"username":"Dyno","id":"155149108183695360","discriminator":"3861","bot":true,"avatar":"5aeb68c29b56b3d92eddb6f46df5051c"},"roles":["241332099666280448","245230583637213184","266733389632110592","280178550496296961"],"mute":false,"joined_at":"2016-10-27T22:47:36.283000+00:00","deaf":false},{"user":{"username":"SirBroBot ⚔","id":"166913295457058817","discriminator":"6030","bot":true,"avatar":"e5b1c2fc9dac54ce6d92225b8b274d0d"},"roles":["269626042686111744","280178550496296961"],"mute":false,"joined_at":"2017-01-14T00:37:37.976000+00:00","deaf":false},{"user":{"username":"Tatsumaki","id":"172002275412279296","discriminator":"8792","bot":true,"avatar":"f5f65755f67ae1dc88d9bb271d0f5bef"},"roles":["245230583637213184","249367374455635978","253599418438254596","266733389632110592","280178550496296961"],"mute":false,"joined_at":"2016-11-19T02:56:55.081000+00:00","deaf":false},{"user":{"username":"YAGPDB.xyz","id":"204255221017214977","discriminator":"8760","bot":true,"avatar":"fcdfbdc43a2a3e836599b0c78a4c17a3"},"roles":["270769959418724363","280178550496296961"],"mute":false,"joined_at":"2017-01-17T04:23:08.979000+00:00","deaf":false},{"user":{"username":"Adonis","id":"216242989041713152","discriminator":"0650","bot":true,"avatar":"eb7252139ad15f4379700f7d4e5cb3dc"},"roles":["266733389632110592","280178550496296961"],"mute":false,"joined_at":"2017-01-06T14:08:54.792000+00:00","deaf":false},{"user":{"username":"funbot","id":"228101986451587072","discriminator":"6874","bot":true,"avatar":null},"roles":[],"mute":false,"joined_at":"2017-05-12T16:33:43.124118+00:00","deaf":false},{"user":{"username":"YAGPDB testing","id":"232658301714825217","discriminator":"1166","bot":true,"avatar":"5d35831179cf9bb65be004bec8045bb6"},"roles":["266728869460049933","280178550496296961"],"mute":false,"joined_at":"2017-01-06T00:45:18.131000+00:00","deaf":false},{"user":{"username":"NotABot","id":"268489917305323520","discriminator":"9791","bot":true,"avatar":null},"roles":[],"mute":false,"joined_at":"2017-05-13T20:07:01.906699+00:00","deaf":false},{"user":{"username":"DEmpire","id":"300419268783112214","discriminator":"5463","bot":true,"avatar":"e14668336c73bdbff2c29a285dbf1369"},"roles":["262513192490631168","280178550496296961"],"nick":null,"mute":false,"joined_at":"2017-04-09T00:01:01.554411+00:00","deaf":false}],"member_count":10,"large":false,"joined_at":"2017-05-13T20:07:01.906699+00:00","id":"183362174188650497","icon":"208492c1c3d8e6d2c24beb6576979b8c","features":[],"explicit_content_filter":2,"emojis":[],"default_message_notifications":0,"channels":[{"type":"text","topic":"AAA","position":4,"permission_overwrites":[{"type":"role","id":"183362174188650497","deny":0,"allow":16384},{"type":"member","id":"232658301714825217","deny":0,"allow":0}],"name":"asdasdaa","last_pin_timestamp":"2016-09-10T09:24:19.580000+00:00","last_message_id":"312628398260224010","is_private":false,"id":"183362174188650497"},{"type":"text","topic":"","position":0,"permission_overwrites":[{"type":"role","id":"270769959418724363","deny":0,"allow":131072}],"name":"another_channel","last_message_id":"312979428688789514","is_private":false,"id":"236574365406199808"},{"type":"text","topic":"","position":1,"permission_overwrites":[],"name":"aaa","last_message_id":"302526022425903104","is_private":false,"id":"241347034983038977"},{"type":"text","topic":null,"position":2,"permission_overwrites":[],"name":"modlog","last_message_id":"274733263875080192","is_private":false,"id":"253968521577365504"},{"type":"text","topic":null,"position":3,"permission_overwrites":[{"type":"role","id":"183362174188650497","deny":0,"allow":1024}],"name":"hello","last_message_id":"300445192258060288","is_private":false,"id":"258024096707641344"},{"type":"text","topic":"asddddddddddd","position":5,"permission_overwrites":[{"type":"role","id":"183362174188650497","deny":0,"allow":16384},{"type":"member","id":"232658301714825217","deny":0,"allow":3072}],"name":"testden","last_message_id":"312628600493047809","is_private":false,"id":"263598866564251649"},{"type":"text","topic":null,"position":6,"permission_overwrites":[],"name":"the-bee-movie","last_message_id":"267818176492863508","is_private":false,"id":"267818041906167808"},{"type":"text","topic":null,"position":7,"permission_overwrites":[],"name":"errs","last_message_id":"307890826283974657","is_private":false,"id":"269534993401774080"},{"type":"text","topic":null,"position":8,"permission_overwrites":[],"name":"joins","last_message_id":"307885094453116928","is_private":false,"id":"269535005389094922"},{"user_limit":0,"type":"voice","position":0,"permission_overwrites":[],"name":"asd","is_private":false,"id":"273135262266294273","bitrate":64000},{"type":"text","topic":null,"position":9,"permission_overwrites":[{"type":"role","id":"294681366522626048","deny":0,"allow":1024},{"type":"role","id":"183362174188650497","deny":0,"allow":0},{"type":"role","id":"294681399024156674","deny":3072,"allow":0}],"name":"ddddd","last_message_id":null,"is_private":false,"id":"285226357854830594"}],"application_id":null,"afk_timeout":300,"afk_channel_id":"273135262266294273"}`)
)
